package pl.edu.pwr.gui;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.hamcrest.Matchers;
import org.junit.Assert;
import org.junit.Test;

public class Iteracja3Test {


	public static String INTENSITY_2_ASCII_10CHARS = "@%#*+=-:. ";
	public static String INTENSITY_2_ASCII_70CHARS = "$@B%8&WM#*oahkbdpqwmZO0QLCJUYXzcvunxrjft/\\|()1{}[]?-_+~<>i!lI;:,\"^`'. ";
	
	@Test
	public void shouldBeProperlyConvertedTo10Characters() {
		shouldBeProperlyConvertedInternal("iteracja3_test1.txt", INTENSITY_2_ASCII_10CHARS);
		shouldBeProperlyScaledInternal("iteracja3_test1.txt", 10, 1);
	}
	
	@Test
	public void shouldBeProperlyConvertedTo70Characters() {
		shouldBeProperlyConvertedInternal("iteracja3_test2.txt", INTENSITY_2_ASCII_70CHARS);
		shouldBeProperlyScaledInternal("iteracja3_test2.txt", 70, 1);
	}
	
	
	private void shouldBeProperlyConvertedInternal(String filename, String expected) {
		InputStream resource = this.getClass().getClassLoader().getResourceAsStream(filename);
		if (resource == null) {
			Assert.fail("Input file not found");
		}
		try (InputStreamReader reader = new InputStreamReader(resource)) {
			try (BufferedReader br = new BufferedReader(reader)) {
				try {
					String line = br.readLine();
					Assert.assertThat(line, Matchers.is(Matchers.equalTo(expected)));
				} catch (IOException e) {
					e.printStackTrace();
					Assert.fail("Something went terribly wrong");
				}
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
	
	@Test
	public void shouldBeProperlyScaledTo80() {
		shouldBeProperlyScaledInternal("eiffel_80.txt", 80, 40);
	}
	
	@Test
	public void shouldBeProperlyScaledTo160() {
		shouldBeProperlyScaledInternal("eiffel_160.txt", 160, 80);
	}
	
	@Test
	public void shouldNotBeScaled() {
		shouldBeProperlyScaledInternal("eiffel.txt", 1024, 512);
	}
		
	@Test
	public void shouldBeProperlyScaledToFullScreen() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		double width = screenSize.getWidth();
		double height = Math.round(width / 2);
		
		shouldBeProperlyScaledInternal("eiffel_full.txt", (int) width, (int) height);
	}
	
	private void shouldBeProperlyScaledInternal(String filename, int expectedWidth, int expectedHeight) {
		InputStream resource = this.getClass().getClassLoader().getResourceAsStream(filename);
		if (resource == null) {
			Assert.fail("Input file not found");
		}
		int lines = 0;
		try (InputStreamReader reader = new InputStreamReader(resource)) {
			try (BufferedReader br = new BufferedReader(reader)) {
				try {
					String line = br.readLine();
					Assert.assertThat(line.length(), Matchers.is(Matchers.equalTo(expectedWidth)));
					lines++;
				} catch (IOException e) {
					e.printStackTrace();
					Assert.fail("Something went terribly wrong");
				}
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		Assert.assertThat(lines, Matchers.is(expectedHeight));
	}
	
}
